@@ -51,6 +51,7 @@ package com.openexchange.ajp13;
 
 import java.io.ByteArrayOutputStream;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicReference;
@@ -359,7 +360,9 @@ public class AJPv13Response {
         final byte[] headers;
         {
             sink.reset();
-            for (final String headerName : servletResponse.getHeaderNames()) {
+            final Iterator<String> iterator = servletResponse.getHeaderNames();
+            for (int i = 0; i < headersSize; i++) {
+                final String headerName = iterator.next();
                 writeHeader(headerName, servletResponse.getHeader(headerName), sink);
             }
             headers = sink.toByteArray();
@@ -453,14 +456,7 @@ public class AJPv13Response {
         /*
          * No need to check against max package size cause it's a static package size of 6
          */
-        if (closeConnection) {
-            return END_RESPONSE_CLOSE;
-        } else if (AJPv13Config.isAJPModJK()) {
-            return END_RESPONSE_REUSE;
-        } else {
-            final boolean reuseConnection = AJPv13Server.getNumberOfOpenAJPSockets() <= AJPv13Config.getAJPMaxNumOfSockets();
-            return reuseConnection ? END_RESPONSE_REUSE : END_RESPONSE_CLOSE;
-        }
+        return closeConnection ? END_RESPONSE_CLOSE : END_RESPONSE_REUSE;
     }
 
     /**
@@ -543,7 +539,7 @@ public class AJPv13Response {
         for (int i = 0; i < formattedCookies.length; i++) {
             final int hdrNameLen;
             {
-                final String hdrName = i == 0 ? STR_SET_COOKIE : new StringBuilder(STR_SET_COOKIE.length() + 1).append(STR_SET_COOKIE).append(
+                final String hdrName = i == 0 ? STR_SET_COOKIE : new com.openexchange.java.StringAllocator(STR_SET_COOKIE.length() + 1).append(STR_SET_COOKIE).append(
                     i + 1).toString();
                 /*
                  * Set-Cookie and Set-Cookie2 is encoded in AJP protocol as an integer value
@@ -768,9 +764,10 @@ public class AJPv13Response {
         if (strLength > 0) {
             final char[] chars = strValue.toCharArray();
             final byte[] bytes = new byte[strLength];
-            for (int i = 0; i < strLength; i++) {
+            System.arraycopy(chars, 0, bytes, 0, strLength);
+            /*for (int i = 0; i < strLength; i++) {
                 bytes[i] = (byte) chars[i];
-            }
+            }*/
             c = writeByteArray(bytes, byteArray, c);
         }
         byteArray[c] = 0;
