@@ -56,6 +56,7 @@ import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
+import java.io.Writer;
 import java.text.DateFormat;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -89,8 +90,10 @@ import com.openexchange.ajp13.servlet.http.HttpDateFormatRegistry;
 import com.openexchange.ajp13.util.CharsetValidator;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.configuration.ServerConfig;
+import com.openexchange.java.AsciiWriter;
 import com.openexchange.java.Charsets;
-import com.openexchange.server.impl.Version;
+import com.openexchange.java.Streams;
+import com.openexchange.version.Version;
 import com.openexchange.session.Session;
 import com.openexchange.tools.regex.MatcherReplacer;
 
@@ -329,11 +332,25 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
 
     @Override
     public void flushBuffer() throws IOException {
-        if (null == writer) {
-            servletOutputStream.flush();
-        } else {
-            writer.flush();
-        }
+        /*-
+         * Since PrintWriter simply delegates flush() invocation to underlying OutputStream,
+         * we can safely call ServletOutputStream.flush() directly.
+         *
+         * See implementation of flush() inside PrintWriter:
+         *    public void flush() {
+         *        try {
+         *        synchronized (lock) {
+         *        ensureOpen()
+         *        out.flush();
+         *        }
+         *        }
+         *        catch (IOException x) {
+         *        trouble = true;
+         *        }
+         *    }
+         */
+        Streams.flush(writer);
+        Streams.flush(servletOutputStream);
     }
 
     /**
@@ -370,11 +387,11 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
                 if (!characterEncoding.equalsIgnoreCase(m.group(2))) {
                     final StringBuilder newContentType = new StringBuilder();
                     final MatcherReplacer mr = new MatcherReplacer(m, contentType);
-                    mr.appendLiteralReplacement(newContentType, new StringBuilder().append(m.group(1)).append(characterEncoding).toString());
+                    mr.appendLiteralReplacement(newContentType, new com.openexchange.java.StringAllocator().append(m.group(1)).append(characterEncoding).toString());
                     while (m.find()) {
                         mr.appendLiteralReplacement(
                             newContentType,
-                            new StringBuilder().append(m.group(1)).append(characterEncoding).toString());
+                            new com.openexchange.java.StringAllocator().append(m.group(1)).append(characterEncoding).toString());
                     }
                     mr.appendTail(newContentType);
                     headers.put(CONTENT_TYPE, Collections.singletonList(newContentType.toString()));
@@ -439,9 +456,11 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
             outputSelection = OUTPUT_WRITER;
         }
         if (bufferSize > 0) {
-            writer = new PrintWriter(new OutputStreamWriter(servletOutputStream, characterEncoding), false);
+            final Writer w = Charsets.isAsciiCharset(characterEncoding) ? new AsciiWriter(servletOutputStream) : new OutputStreamWriter(servletOutputStream, characterEncoding);
+            writer = new PrintWriter(w, false);
         } else {
-            writer = new PrintWriter(new OutputStreamWriter(servletOutputStream, characterEncoding), false);
+            final Writer w = Charsets.isAsciiCharset(characterEncoding) ? new AsciiWriter(servletOutputStream) : new OutputStreamWriter(servletOutputStream, characterEncoding);
+            writer = new PrintWriter(w, false);
         }
         return writer;
     }
@@ -593,7 +612,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         if (url == null) {
             return null;
         } else if (groupwareSessionId == null && httpSessionId == null) {
-            return url.indexOf('?') == -1 ? new StringBuilder(url).append("?jvm=").append(AJPv13Config.getJvmRoute()).toString() : url;
+            return url.indexOf('?') == -1 ? new com.openexchange.java.StringAllocator(url).append("?jvm=").append(AJPv13Config.getJvmRoute()).toString() : url;
         }
         String path = url;
         String query = "";
@@ -608,7 +627,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
             anchor = path.substring(pound);
             path = path.substring(0, pound);
         }
-        final StringBuilder sb = new StringBuilder(path);
+        final com.openexchange.java.StringAllocator sb = new com.openexchange.java.StringAllocator(path);
         if (httpSessionId != null && sb.length() > 0) {
             sb.append('/');
             sb.append(AJPv13RequestHandler.JSESSIONID_URI);
@@ -702,7 +721,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
                 // Already existing; decide which one to keep or to merge
                 // By now: Keep the newer one (cookies is a LinkedHashSet that keeps order)
                 /*-
-                 * 
+                 *
                 if (null != prev) {
                     if (0 == prev.getMaxAge()) {
                         // First indeciates delete
@@ -712,7 +731,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
                         }
                     } else {
                         // First i
-                        
+
                     }
                 }
                 */
@@ -963,12 +982,12 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         String errorMsgStr = ERROR_PAGE_TEMPL;
         errorMsgStr = errorMsgStr.replaceAll("#STATUS_CODE#", String.valueOf(this.status)).replaceAll(
             "#STATUS_MSG#",
-            Matcher.quoteReplacement(this.statusMsg)).replaceFirst("#STATUS_DESC#", desc);
+            com.openexchange.java.Strings.quoteReplacement(this.statusMsg)).replaceFirst("#STATUS_DESC#", com.openexchange.java.Strings.quoteReplacement(desc));
         synchronized (HEADER_DATE_FORMAT) {
             errorMsgStr = errorMsgStr.replaceFirst("#DATE#", HEADER_DATE_FORMAT.format(new Date(System.currentTimeMillis())));
         }
-        errorMsgStr = errorMsgStr.replaceFirst("#VERSION#", Version.getVersionString());
-        setContentType(new StringBuilder("text/html; charset=").append(getCharacterEncoding()).toString());
+        errorMsgStr = errorMsgStr.replaceFirst("#VERSION#", Version.getInstance().getVersionString());
+        setContentType(new com.openexchange.java.StringAllocator("text/html; charset=").append(getCharacterEncoding()).toString());
         final byte[] errormessage = errorMsgStr.getBytes(Charsets.forName(getCharacterEncoding()));
         setContentLength(errormessage.length);
         return errormessage;
@@ -1000,12 +1019,12 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         synchronized (HEADER_DATE_FORMAT) {
             errorMsgStr = errorMsgStr.replaceFirst("#DATE#", HEADER_DATE_FORMAT.format(new Date(System.currentTimeMillis())));
         }
-        errorMsgStr = errorMsgStr.replaceFirst("#VERSION#", Version.getVersionString());
+        errorMsgStr = errorMsgStr.replaceFirst("#VERSION#", Version.getInstance().getVersionString());
         String encoding = getCharacterEncoding();
         if (null == encoding) {
             encoding = "UTF-8";
         }
-        setContentType(new StringBuilder("text/html; charset=").append(encoding).toString());
+        setContentType(new com.openexchange.java.StringAllocator("text/html; charset=").append(encoding).toString());
         final byte[] errormessage = errorMsgStr.getBytes(Charsets.forName(encoding));
         setContentLength(errormessage.length);
         return errormessage;
