@@ -50,6 +50,7 @@
 package com.openexchange.ajp13.coyote;
 
 import static com.openexchange.ajp13.AJPv13Response.writeHeaderSafe;
+import static com.openexchange.ajp13.AJPv13Utility.decodeUrl;
 import static com.openexchange.tools.servlet.http.Cookies.extractDomainValue;
 import static com.openexchange.tools.servlet.http.Cookies.getDomainValue;
 import java.io.ByteArrayOutputStream;
@@ -60,7 +61,6 @@ import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
 import java.net.Socket;
-import java.net.URLDecoder;
 import java.net.URLEncoder;
 import java.nio.charset.UnsupportedCharsetException;
 import java.util.List;
@@ -100,6 +100,7 @@ import com.openexchange.log.LogProperties;
 import com.openexchange.log.Props;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
+import com.openexchange.tools.exceptions.ExceptionUtils;
 import com.openexchange.tools.servlet.UploadServletException;
 import com.openexchange.tools.servlet.http.Cookies;
 import com.openexchange.tools.stream.UnsynchronizedByteArrayOutputStream;
@@ -714,9 +715,9 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                  * Gather logging info
                  */
                 final Props properties = LogProperties.getLogProperties();
-                properties.put("com.openexchange.ajp13.threadName", thread.getName());
-                properties.put("com.openexchange.ajp13.remotePort", Integer.valueOf(socket.getPort()));
-                properties.put("com.openexchange.ajp13.remoteAddress", socket.getInetAddress().getHostAddress());
+                properties.put(LogProperties.Name.AJP_THREAD_NAME, thread.getName());
+                properties.put(LogProperties.Name.AJP_REMOTE_PORT, Integer.valueOf(socket.getPort()));
+                properties.put(LogProperties.Name.AJP_REMOTE_ADDRESS, socket.getInetAddress().getHostAddress());
             }
             final boolean debug = LOG.isDebugEnabled();
             try {
@@ -824,7 +825,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 } catch (final IllegalStateException stateException) {
                     /*-
                      * Max. number of HTTP sessions reached.
-                     * 
+                     *
                      * Status code (503) indicating that the HTTP server is
                      * temporarily overloaded, and unable to handle the request.
                      */
@@ -837,7 +838,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 } catch (final Throwable t) {
                     // 400 - Bad Request
                     {
-                        final StringBuilder sb = new StringBuilder(512);
+                        final com.openexchange.java.StringAllocator sb = new com.openexchange.java.StringAllocator(512);
                         sb.append("400 - Bad Request: Error preparing forward-request: ").append(t.getClass().getName());
                         sb.append(" message=").append(t.getMessage());
                         if (debug) {
@@ -942,7 +943,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     // Ignore
                 } catch (final Throwable t) {
                     ExceptionUtils.handleThrowable(t);
-                    final StringBuilder tmp = new StringBuilder(128).append("Error processing request: ");
+                    final com.openexchange.java.StringAllocator tmp = new com.openexchange.java.StringAllocator(128).append("Error processing request: ");
                     appendRequestInfo(tmp);
                     LOG.error(tmp.toString(), t);
                     // 500 - Internal Server Error
@@ -973,7 +974,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     error = true;
                 } catch (final Throwable t) {
                     ExceptionUtils.handleThrowable(t);
-                    final StringBuilder tmp = new StringBuilder(128).append("Error processing request: ");
+                    final com.openexchange.java.StringAllocator tmp = new com.openexchange.java.StringAllocator(128).append("Error processing request: ");
                     appendRequestInfo(tmp);
                     LOG.error(tmp.toString(), t);
                     error = true;
@@ -1009,10 +1010,10 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
 
     /**
      * Appends request information.
-     * 
+     *
      * @param builder The builder to append to
      */
-    protected void appendRequestInfo(final StringBuilder builder) {
+    protected void appendRequestInfo(final com.openexchange.java.StringAllocator builder) {
         builder.append("request-URI=''");
         builder.append(request.getRequestURI());
         builder.append("'', query-string=''");
@@ -1255,65 +1256,67 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         }
 
         // Decode headers
-        final int hCount = requestHeaderMessage.getInt();
-        for (int i = 0; i < hCount; i++) {
-            final String hName;
-            // Header names are encoded as either an integer code starting
-            // with 0xA0, or as a normal string (in which case the first
-            // two bytes are the length).
-            int isc = requestHeaderMessage.peekInt();
-            int hId = isc & 0xFF;
-            isc &= 0xFF00;
-            if (0xA000 == isc) {
-                requestHeaderMessage.getInt(); // To advance the read position
-                hName = Constants.headerTransArray[hId - 1];
-            } else {
-                // reset hId -- if the header currently being read
-                // happens to be 7 or 8 bytes long, the code below
-                // will think it's the content-type header or the
-                // content-length header - SC_REQ_CONTENT_TYPE=7,
-                // SC_REQ_CONTENT_LENGTH=8 - leading to unexpected
-                // behaviour. see bug 5861 for more information.
-                hId = -1;
-                hName = requestHeaderMessage.getString(temp);
-            }
-            final String hValue = requestHeaderMessage.getString(temp);
-            /*
-             * Check for "Content-Length" and "Content-Type" headers
-             */
-            if (hId == Constants.SC_REQ_CONTENT_LENGTH || (hId == -1 && hName.equalsIgnoreCase("Content-Length"))) {
-                /*
-                 * Read the content-length header, so set it
-                 */
-                final long cl = Long.parseLong(hValue);
-                if (cl < Integer.MAX_VALUE) {
-                    request.setContentLength(cl);
+        {
+            final int hCount = requestHeaderMessage.getInt();
+            for (int i = 0; i < hCount; i++) {
+                final String hName;
+                // Header names are encoded as either an integer code starting
+                // with 0xA0, or as a normal string (in which case the first
+                // two bytes are the length).
+                int isc = requestHeaderMessage.peekInt();
+                int hId = isc & 0xFF;
+                isc &= 0xFF00;
+                if (0xA000 == isc) {
+                    requestHeaderMessage.getInt(); // To advance the read position
+                    hName = Constants.headerTransArray[hId - 1];
+                } else {
+                    // reset hId -- if the header currently being read
+                    // happens to be 7 or 8 bytes long, the code below
+                    // will think it's the content-type header or the
+                    // content-length header - SC_REQ_CONTENT_TYPE=7,
+                    // SC_REQ_CONTENT_LENGTH=8 - leading to unexpected
+                    // behaviour. see bug 5861 for more information.
+                    hId = -1;
+                    hName = requestHeaderMessage.getString(temp);
                 }
-            } else if (hId == Constants.SC_REQ_CONTENT_TYPE || (hId == -1 && hName.equalsIgnoreCase("Content-Type"))) {
+                final String hValue = requestHeaderMessage.getString(temp);
                 /*
-                 * Read the content-type header, so set it
+                 * Check for "Content-Length" and "Content-Type" headers
                  */
-                try {
-                    request.setContentType(hValue);
-                } catch (final AJPv13Exception e) {
-                    response.setStatus(403);
-                    error = true;
-                }
-            } else if (hId == Constants.SC_REQ_COOKIE || (hId == -1 && hName.equalsIgnoreCase("Cookie"))) {
-                /*
-                 * Read a cookie, so set it
-                 */
-                try {
-                    request.setCookies(CookieParser.parseCookieHeader(hValue));
-                } catch (final AJPv13Exception e) {
-                    response.setStatus(403);
-                    error = true;
-                }
-            } else {
-                try {
-                    request.setHeader(hName, hValue, false);
-                } catch (final AJPv13Exception e) {
-                    // Cannot occur
+                if (hId == Constants.SC_REQ_CONTENT_LENGTH || (hId == -1 && hName.equalsIgnoreCase("Content-Length"))) {
+                    /*
+                     * Read the content-length header, so set it
+                     */
+                    final long cl = Long.parseLong(hValue);
+                    if (cl < Integer.MAX_VALUE) {
+                        request.setContentLength(cl);
+                    }
+                } else if (hId == Constants.SC_REQ_CONTENT_TYPE || (hId == -1 && hName.equalsIgnoreCase("Content-Type"))) {
+                    /*
+                     * Read the content-type header, so set it
+                     */
+                    try {
+                        request.setContentType(hValue);
+                    } catch (final AJPv13Exception e) {
+                        response.setStatus(403);
+                        error = true;
+                    }
+                } else if (hId == Constants.SC_REQ_COOKIE || (hId == -1 && hName.equalsIgnoreCase("Cookie"))) {
+                    /*
+                     * Read a cookie, so set it
+                     */
+                    try {
+                        request.setCookies(CookieParser.parseCookieHeader(hValue));
+                    } catch (final AJPv13Exception e) {
+                        response.setStatus(403);
+                        error = true;
+                    }
+                } else {
+                    try {
+                        request.setHeader(hName, hValue, false);
+                    } catch (final AJPv13Exception e) {
+                        // Cannot occur
+                    }
                 }
             }
         }
@@ -1325,7 +1328,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
             if (null != echoHeaderName) {
                 final String echoValue = request.getHeader(echoHeaderName);
                 if (null != echoValue) {
-                    LogProperties.putLogProperty("com.openexchange.ajp13.requestId", echoValue);
+                    LogProperties.putLogProperty(LogProperties.Name.AJP_REQUEST_ID, echoValue);
                 }
             }
         }
@@ -1385,13 +1388,11 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
             case Constants.SC_A_QUERY_STRING: {
                 final String queryString = requestHeaderMessage.getString(temp);
                 request.setQueryString(queryString);
-                try {
+                {
                     parseQueryString(queryString);
                     if (DEBUG && isEASPingCommand()) {
                         LOG.debug("Incoming long-running EAS ping request.");
                     }
-                } catch (final UnsupportedEncodingException e) {
-                    throw new IllegalStateException(e.getMessage(), e);
                 }
             }
                 break;
@@ -1448,27 +1449,29 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         /*
          * Check for a full URI (including protocol://host:port/)
          */
-        final String requestURI = request.getRequestURI();
-        if (requestURI.regionMatches(true, 0, "http", 0, 4)) {
-            final int pos = requestURI.indexOf("://", 4);
-            int slashPos = -1;
-            if (pos != -1) {
-                slashPos = requestURI.indexOf('/', pos + 3);
-                if (slashPos == -1) {
-                    slashPos = requestURI.length();
-                    /*
-                     * Set URI as "/"
-                     */
-                    request.setRequestURI("/");
-                } else {
-                    request.setRequestURI(requestURI.substring(slashPos));
-                }
-                final String host = request.getHeader("host");
-                if (null != host) {
-                    try {
-                        request.setHeader("host", host.substring(pos + 3, slashPos), false);
-                    } catch (final AJPv13Exception e) {
-                        // Cannot occur
+        {
+            final String requestURI = request.getRequestURI();
+            if (requestURI.regionMatches(true, 0, "http", 0, 4)) {
+                final int pos = requestURI.indexOf("://", 4);
+                int slashPos = -1;
+                if (pos != -1) {
+                    slashPos = requestURI.indexOf('/', pos + 3);
+                    if (slashPos == -1) {
+                        slashPos = requestURI.length();
+                        /*
+                         * Set URI as "/"
+                         */
+                        request.setRequestURI("/");
+                    } else {
+                        request.setRequestURI(requestURI.substring(slashPos));
+                    }
+                    final String host = request.getHeader("host");
+                    if (null != host) {
+                        try {
+                            request.setHeader("host", host.substring(pos + 3, slashPos), false);
+                        } catch (final AJPv13Exception e) {
+                            // Cannot occur
+                        }
                     }
                 }
             }
@@ -1476,9 +1479,11 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         /*
          * Parse host header
          */
-        final String host = request.getHeader("host");
-        if (null != host) {
-            parseHost(host);
+        {
+            final String host = request.getHeader("host");
+            if (null != host) {
+                parseHost(host);
+            }
         }
         /*-
          * Parsing done
@@ -1490,12 +1495,16 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         {
             final Props properties = LogProperties.getLogProperties();
             if (LogProperties.isEnabled()) {
-                properties.put("com.openexchange.ajp13.requestURI", request.getRequestURI());
-                properties.put("com.openexchange.ajp13.servletPath", request.getServletPath());
-                properties.put("com.openexchange.ajp13.pathInfo", request.getPathInfo());
+                properties.put(LogProperties.Name.AJP_REQUEST_URI, request.getRequestURI());
+                properties.put(LogProperties.Name.AJP_SERVLET_PATH, request.getServletPath());
+                properties.put(LogProperties.Name.AJP_PATH_INFO, request.getPathInfo());
+                final String action = request.getParameter("action");
+                if (null != action) {
+                    properties.put(LogProperties.Name.AJAX_ACTION, action);
+                }
             }
-            properties.put("com.openexchange.ajp13.requestIp", request.getRemoteAddr());
-            properties.put("com.openexchange.ajp13.serverName", serverName);
+            properties.put(LogProperties.Name.AJP_REQUEST_IP, request.getRemoteAddr());
+            properties.put(LogProperties.Name.AJP_SERVER_NAME, serverName);
         }
         /*
          * Set proper JSESSIONID cookie and pre-create associated HTTP session
@@ -1575,7 +1584,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
             for (int i = length - 1; i > colonPos; i--) {
                 final int charValue = host.charAt(i);
                 /*-
-                 * 
+                 *
                 if (charValue == -1) {
                     // Invalid character
                     error = true;
@@ -1584,7 +1593,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     response.setStatus(400);
                     break;
                 }
-                 * 
+                 *
                  */
                 port = port + (charValue * mult);
                 mult = 10 * mult;
@@ -1599,9 +1608,8 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
      * Parses a query string and puts resulting parameters into given servlet request.
      *
      * @param queryStr The query string to be parsed
-     * @throws UnsupportedEncodingException If charset provided by servlet request is not supported
      */
-    private void parseQueryString(final String queryStr) throws UnsupportedEncodingException {
+    private void parseQueryString(final String queryStr) {
         final String[] paramsNVPs = PATTERN_SPLIT.split(queryStr, 0);
         String charEnc = request.getCharacterEncoding();
         if (null == charEnc) {
@@ -1613,7 +1621,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 // Look-up character '='
                 final int pos = paramsNVP.indexOf('=');
                 if (pos >= 0) {
-                    request.setParameter(paramsNVP.substring(0, pos), URLDecoder.decode(paramsNVP.substring(pos + 1), charEnc));
+                    request.setParameter(paramsNVP.substring(0, pos), decodeUrl(paramsNVP.substring(pos + 1), charEnc));
                 } else {
                     request.setParameter(paramsNVP, "");
                 }
@@ -1719,7 +1727,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                              * Different JVM route detected -> Discard
                              */
                             if (DEBUG) {
-                                LOG.debug(new StringBuilder("\n\tDifferent JVM route detected. Removing JSESSIONID cookie: ").append(id));
+                                LOG.debug(new com.openexchange.java.StringAllocator("\n\tDifferent JVM route detected. Removing JSESSIONID cookie: ").append(id));
                             }
                             current.setPath("/");
                             final String domain = extractDomainValue(id);
@@ -1745,7 +1753,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                              * Invalid cookie
                              */
                             if (DEBUG) {
-                                LOG.debug(new StringBuilder("\n\tExpired or invalid cookie -> Removing JSESSIONID cookie: ").append(current.getValue()));
+                                LOG.debug(new com.openexchange.java.StringAllocator("\n\tExpired or invalid cookie -> Removing JSESSIONID cookie: ").append(current.getValue()));
                             }
                             current.setPath("/");
                             final String domain = extractDomainValue(id);
@@ -1764,7 +1772,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                             continue NextCookie;
                         }
                         jsessionIDCookie = current;
-                        LogProperties.putLogProperty("com.openexchange.ajp13.httpSession", id);
+                        LogProperties.putLogProperty(LogProperties.Name.AJP_HTTP_SESSION, id);
                         jsessionIDCookie.setSecure(request.isSecure() || (forceHttps && !Cookies.isLocalLan(request)));
                         httpSessionCookie = jsessionIDCookie;
                         httpSessionJoined = true;
@@ -1777,7 +1785,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                              * But this host defines a JVM route
                              */
                             if (DEBUG) {
-                                LOG.debug(new StringBuilder("\n\tMissing JVM route in JESSIONID cookie").append(current.getValue()));
+                                LOG.debug(new com.openexchange.java.StringAllocator("\n\tMissing JVM route in JESSIONID cookie").append(current.getValue()));
                             }
                             current.setPath("/");
                             final String domain = extractDomainValue(id);
@@ -1803,7 +1811,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                              * Invalid cookie
                              */
                             if (DEBUG) {
-                                LOG.debug(new StringBuilder("\n\tExpired or invalid cookie -> Removing JSESSIONID cookie: ").append(current.getValue()));
+                                LOG.debug(new com.openexchange.java.StringAllocator("\n\tExpired or invalid cookie -> Removing JSESSIONID cookie: ").append(current.getValue()));
                             }
                             current.setPath("/");
                             final String domain = extractDomainValue(id);
@@ -1822,7 +1830,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                             continue NextCookie;
                         }
                         jsessionIDCookie = current;
-                        LogProperties.putLogProperty("com.openexchange.ajp13.httpSession", id);
+                        LogProperties.putLogProperty(LogProperties.Name.AJP_HTTP_SESSION, id);
                         jsessionIDCookie.setSecure(request.isSecure() || (forceHttps && !Cookies.isLocalLan(request)));
                         httpSessionCookie = jsessionIDCookie;
                         httpSessionJoined = true;
@@ -1848,7 +1856,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         /*
          * Create a new unique id
          */
-        final StringBuilder jsessionIDVal = new StringBuilder(HttpSessionManagement.getNewUniqueId());
+        final com.openexchange.java.StringAllocator jsessionIDVal = new com.openexchange.java.StringAllocator(HttpSessionManagement.getNewUniqueId());
         final String jvmRoute = AJPv13Config.getJvmRoute();
         final String domain = getDomainValue(serverName);
         if ((jvmRoute != null) && (jvmRoute.length() > 0)) {
@@ -1859,7 +1867,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         }
         final String id = jsessionIDVal.toString();
         final Cookie jsessionIDCookie = newJsessionIdCookie(id, domain);
-        LogProperties.putLogProperty("com.openexchange.ajp13.httpSession", id);
+        LogProperties.putLogProperty(LogProperties.Name.AJP_HTTP_SESSION, id);
         jsessionIDCookie.setSecure(request.isSecure() || (forceHttps && !Cookies.isLocalLan(request)));
         httpSessionCookie = jsessionIDCookie;
         httpSessionJoined = false;
@@ -1895,7 +1903,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
             join = false;
         }
         final Cookie jsessionIDCookie = newJsessionIdCookie(jsessionIdVal, domain);
-        LogProperties.putLogProperty("com.openexchange.ajp13.httpSession", jsessionIdVal);
+        LogProperties.putLogProperty(LogProperties.Name.AJP_HTTP_SESSION, jsessionIdVal);
         jsessionIDCookie.setSecure(request.isSecure() || (forceHttps && !Cookies.isLocalLan(request)));
         httpSessionCookie = jsessionIDCookie;
         httpSessionJoined = join;
@@ -1951,11 +1959,13 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         /*-
          * Check for echo header presence
          */
-        final String echoHeaderName = AJPv13Response.getEchoHeaderName();
-        if (null != echoHeaderName) {
-            final String echoValue = request.getHeader(echoHeaderName);
-            if (null != echoValue) {
-                response.setHeader(echoHeaderName, urlEncode(echoValue));
+        {
+            final String echoHeaderName = AJPv13Response.getEchoHeaderName();
+            if (null != echoHeaderName) {
+                final String echoValue = request.getHeader(echoHeaderName);
+                if (null != echoValue) {
+                    response.setHeader(echoHeaderName, urlEncode(echoValue));
+                }
             }
         }
         /*
@@ -2361,7 +2371,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
 
     } // End of class
 
-    private static void appendStackTrace(final StackTraceElement[] trace, final StringBuilder sb) {
+    private static void appendStackTrace(final StackTraceElement[] trace, final com.openexchange.java.StringAllocator sb) {
         if (null == trace) {
             return;
         }
@@ -2391,7 +2401,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
 
     /**
      * Generates URL-encoding of specified text.
-     * 
+     *
      * @param text The text
      * @return The URL-encoded text
      */
