@@ -208,7 +208,7 @@ public class AJPv13TaskWatcher {
                         final int numProcessing = countProcessing.get();
                         if (numProcessing > 0 && countExceeded.get() == numProcessing) {
                             final String delimStr = "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
-                            log.error(new StringBuilder(128 + delimStr.length()).append(delimStr).append(
+                            log.error(new com.openexchange.java.StringAllocator(128 + delimStr.length()).append(delimStr).append(
                                 "AJP-Watcher's run done: SYSTEM DEADLOCK DETECTED!").append(" Going to stop and re-initialize system").append(
                                 delimStr).toString());
                             /*
@@ -313,10 +313,10 @@ public class AJPv13TaskWatcher {
                 final Map<String, Object> taskProperties;
                 {
                     final Props taskProps = LogProperties.optLogProperties(task.getThread());
-                    taskProperties = null == taskProps ? null : taskProps.getMap();
+                    taskProperties = null == taskProps ? null : taskProps.asMap();
                 }
                 if (null == taskProperties) {
-                    final StringBuilder logBuilder = new StringBuilder(196).append("AJP Listener \"").append(task.getThreadName());
+                    final com.openexchange.java.StringAllocator logBuilder = new com.openexchange.java.StringAllocator(196).append("AJP Listener \"").append(task.getThreadName());
                     logBuilder.append("\" exceeds max. running time of ").append(AJPv13Config.getAJPWatcherMaxRunningTime());
                     logBuilder.append("msec -> Processing time: ").append(System.currentTimeMillis() - task.getProcessingStartTime());
                     logBuilder.append("msec");
@@ -324,7 +324,7 @@ public class AJPv13TaskWatcher {
                     appendStackTrace(task.getStackTrace(), logBuilder);
                     log.info(logBuilder);
                 } else {
-                    final StringBuilder logBuilder = new StringBuilder(512);
+                    final com.openexchange.java.StringAllocator logBuilder = new com.openexchange.java.StringAllocator(2048);
                     final Map<String, String> sorted = new TreeMap<String, String>();
                     for (final Entry<String, Object> entry : taskProperties.entrySet()) {
                         final String propertyName = entry.getKey();
@@ -351,11 +351,14 @@ public class AJPv13TaskWatcher {
             }
         }
 
-        private static void appendStackTrace(final StackTraceElement[] trace, final StringBuilder sb) {
+        private static final int MAX_STACK_TRACE_ELEMENTS = 1000;
+
+        private static void appendStackTrace(final StackTraceElement[] trace, final com.openexchange.java.StringAllocator sb) {
             if (null == trace) {
                 return;
             }
-            for (final StackTraceElement ste : trace) {
+            for (int i = 0; i < trace.length && i < MAX_STACK_TRACE_ELEMENTS; i++) {
+                final StackTraceElement ste = trace[i];
                 final String className = ste.getClassName();
                 if (null != className) {
                     sb.append("\tat ").append(className).append('.').append(ste.getMethodName());
